#include <glib.h>
#include <pan/base/db.h>
#include <pan/base/db-add.h>
#include <pan/base/article-thread.h>
#include <gmime/gmime.h>
#include <gmime/md5-utils.h>
#include <pan/base/debug.h>
#include <pan/base/group-private.h>
#include <pan/base/article.h>
#include <pan/base/article-private.h>

/**
 * add_article:
 * @gid: group id.
 * @gid: server id.
 * @article: article_add_s *.
 *
 * Adds one article overview to the db.  It DOES NOT use a transaaction.
 *
 * Returns: gint64 id of inserted article or %0 on fail.
 */
gint64 add_article(gint32 gid, gint32 sid, const article_add_s *article)
{
	g_return_val_if_fail(article!=NULL,0);
	debug_enter("add_article");
	
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res;
	char *errmsg;
	char *subj;
	PString st=PSTRING_INIT;
	int part=0,parts=0,len;
	gint64 aid=0,base_id=0;
	gint32 gsid=0,nsid=0,authid=0,psid=0;
	char *pre,*norm,*post,*mid,hash[16];
	InternetAddressList *addr=internet_address_parse_string(article->author.str);
	const char *aa,*ar;
	gboolean exists=FALSE,exists_base=FALSE;
	aa=ar="";
	pre=norm=post=NULL;
	
	if(addr && addr->address)
	{
		ar=addr->address->name;
		aa=addr->address->value.addr;
	}
	
	mid=g_mime_utils_decode_message_id(article->msg_id.str);
	md5_get_digest(mid,strlen(mid),hash);
	
	subj=g_mime_utils_8bit_header_decode(article->subject.str);
	g_assert(subj!=NULL);
	st=pstring_shallow(subj,-1);
	if(!normalize_subject(&st,&pre,&norm,&post))
	{
		// QUESTION: do we need to log the error here?
		g_free(subj);
		g_free(mid);
		internet_address_list_destroy(addr);
		debug_exit("add_article");
		return 0;
	}
	g_free(subj);
	subj=NULL;
	len=strlen(norm);
	st=PSTRING_INIT;
	
	if(*post)
	{
		char *s=post+1;
		while(*s && !g_ascii_isdigit(*s))
			++s;
		part=atoi(s);
		while(*s && *s!='/')
			++s;
		while(*s && !g_ascii_isdigit(*s))
			++s;
		parts=atoi(s);
	}
	/* if not a multipart yet, AND if it's a big message, AND
	   it's either in one of the pictures/fan/sex groups or it
	   has commonly-used image names in the subject, guess it's
	   a single-part binary */
	if(!parts && article->lines>400 && 
		( ( article->xref.str!=NULL && 
			( g_strstr_len(article->xref.str, article->xref.len, "binaries")
				|| g_strstr_len(article->xref.str, article->xref.len, "fan")
				|| g_strstr_len(article->xref.str, article->xref.len, "mag")
				|| g_strstr_len(article->xref.str, article->xref.len, "sex") 
			)
		  )
			|| ((pan_strstr(norm,".jpg") || pan_strstr(norm,".JPG")
				|| pan_strstr(norm,".jpeg") || pan_strstr(norm,".JPEG")
				|| pan_strstr(norm,".gif")  || pan_strstr(norm,".GIF")
				|| pan_strstr(norm,".tiff") || pan_strstr(norm,".TIFF")))
		) )
		part=parts=1;
	/* but if it's starting the subject with "Re:" and doesn't
	   have many lines, it's probably a followup to a part, rather
	   than an actual part. */
	if(parts && *pre && article->lines<100)
		part=parts=0;
	
	if(part==0)
		parts=0;
	if(parts<part)
		part=parts=0;
	if(parts>1)
	{
		char *temp,*x=g_strstr_len(post+1,8,"/");
		temp=g_strdup_printf("%c%s",*post,x);
		g_free(post);
		post=temp;
	}
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO norm_subject(subject) VALUES(?);",
							-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,pre,-1,SQLITE_STATIC);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	if(sqlite3_changes(db)==0)
	{
		sq_res=sqlite3_prepare(db,"SELECT id FROM norm_subject WHERE subject=?;",
								-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_text(stmt,1,pre,-1,SQLITE_STATIC);
		sqlite3_step(stmt);
		psid=sqlite3_column_int(stmt,0);
		sqlite3_finalize(stmt);	
	} else
	{
		psid=sqlite3_last_insert_rowid(db);
	}

	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO norm_subject(subject) VALUES(?);",
							-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,norm,-1,SQLITE_STATIC);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	if(sqlite3_changes(db)==0)
	{
		sq_res=sqlite3_prepare(db,"SELECT id FROM norm_subject WHERE subject=?;",
								-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_text(stmt,1,norm,-1,SQLITE_STATIC);
		sqlite3_step(stmt);
		nsid=sqlite3_column_int(stmt,0);
		sqlite3_finalize(stmt);	
	} else
	{
		nsid=sqlite3_last_insert_rowid(db);
	}
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO authors(real,address) VALUES(?,?);",
							-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,ar,-1,SQLITE_STATIC);
	sqlite3_bind_text(stmt,2,aa,-1,SQLITE_STATIC);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	if(sqlite3_changes(db)==0)
	{
		sq_res=sqlite3_prepare(db,"SELECT id FROM authors WHERE real=? AND address=?;",
								-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_text(stmt,1,ar,-1,SQLITE_STATIC);
		sqlite3_bind_text(stmt,2,aa,-1,SQLITE_STATIC);
		sqlite3_step(stmt);
		authid=sqlite3_column_int(stmt,0);
		sqlite3_finalize(stmt);	
	} else
	{
		authid=sqlite3_last_insert_rowid(db);
	}
	
	// insert a dummy base article if more than one part
	if(parts>1)
	{
		sq_res=sqlite3_prepare(db,"SELECT id FROM article WHERE norm_subj=? "
									"AND pre_subj=? AND hash ISNULL AND author=? "
									"AND parts=? AND post_subj=?;",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_text(stmt,5,post,-1,SQLITE_STATIC);
		sqlite3_bind_int(stmt,2,psid);
		sqlite3_bind_int(stmt,1,nsid);
		sqlite3_bind_int(stmt,3,authid);
		sqlite3_bind_int(stmt,4,parts);
		if(sqlite3_step(stmt)==SQLITE_ROW)
		{
			exists_base=TRUE;
			base_id=sqlite3_column_int64(stmt,0);
		}
		sqlite3_finalize(stmt);
		// create the base if it didn't exist
		if G_UNLIKELY(base_id==0)
		{
			sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO article(hash, message_id, "
										"date, lines, author, byte_qty, parts, "
										"pre_subj, norm_subj, post_subj) "
										"VALUES(?7,?1,?2,?3,?4,?5,?6,?8,?9,?10);",
										-1,&stmt,NULL);
			SQL_PREP_VALID(sq_res,db);
			sqlite3_bind_text(stmt,1,"",0,SQLITE_STATIC);
			sqlite3_bind_null(stmt,7);
			sqlite3_bind_int(stmt,8,psid);
			sqlite3_bind_text(stmt,10,post,-1,SQLITE_STATIC);
			sqlite3_bind_int(stmt,2,article->date);
			sqlite3_bind_int(stmt,3,0);
			sqlite3_bind_int(stmt,4,authid);
			sqlite3_bind_int(stmt,5,article->byte_qty);
			sqlite3_bind_int(stmt,6,parts);
			sqlite3_bind_int(stmt,9,nsid);
			sqlite3_step(stmt);
			sqlite3_finalize(stmt);
			base_id=sqlite3_last_insert_rowid(db);
		}
		else if(part==1 && exists_base)
		{
			// FIXME: only update date, have db add lines & byte_qty
			// base should have their sum
			sq_res=sqlite3_prepare(db,"UPDATE article SET date=?, lines=?, "
										"byte_qty=? WHERE id=?;",-1,&stmt,NULL);
			SQL_PREP_VALID(sq_res,db);
			sqlite3_bind_int(stmt,1,article->date);
			sqlite3_bind_int(stmt,2,article->lines);
			sqlite3_bind_int(stmt,3,article->byte_qty);
			sqlite3_bind_int64(stmt,4,base_id);
			sqlite3_step(stmt);
			sqlite3_finalize(stmt);
		}
	}
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO article(hash, message_id, "
								"date, lines, author, byte_qty, parts, "
								"pre_subj, norm_subj, post_subj,part,base_id) "
								"VALUES(?7,?1,?2,?3,?4,?5,?6,?8,?9,?10,?11,?12);",
								-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,mid,-1,SQLITE_STATIC);
	sqlite3_bind_blob(stmt,7,hash,16,SQLITE_STATIC);
	sqlite3_bind_text(stmt,10,post,-1,SQLITE_STATIC);
	sqlite3_bind_int(stmt,2,article->date);
	sqlite3_bind_int(stmt,3,article->lines);
	sqlite3_bind_int(stmt,4,authid);
	sqlite3_bind_int(stmt,5,article->byte_qty);
	sqlite3_bind_int(stmt,6,parts);
	sqlite3_bind_int(stmt,8,psid);
	sqlite3_bind_int(stmt,9,nsid);
	sqlite3_bind_int(stmt,11,part);
	if(base_id!=0)
		sqlite3_bind_int64(stmt,12,base_id);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	if G_LIKELY(sqlite3_changes(db)!=0)
		aid=sqlite3_last_insert_rowid(db);
	else
	{
		exists=TRUE;
		sq_res=sqlite3_prepare(db,"SELECT id FROM article WHERE hash=?;",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_blob(stmt,1,hash,16,SQLITE_STATIC);
		sqlite3_step(stmt);
		aid=sqlite3_column_int64(stmt,0);
		sqlite3_finalize(stmt);
	}
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO group_article(group_id,article_id) "
								"VALUES(?,?);",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_int(stmt,1,gid);
	sqlite3_bind_int64(stmt,2,(parts>1)?base_id:aid);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	
	sq_res=sqlite3_prepare(db,"SELECT id FROM group_server WHERE group_id=? AND server_id=?;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_int(stmt,1,gid);
	sqlite3_bind_int(stmt,2,sid);
	sqlite3_step(stmt);
	gsid=sqlite3_column_int(stmt,0);
	sqlite3_finalize(stmt);
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO g_s_a(group_server_id,article_id,uid) "
								"VALUES(?,?,?);",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_int(stmt,1,gsid);
	sqlite3_bind_int64(stmt,2,aid);
	sqlite3_bind_int(stmt,3,article->uid);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	
	if(!exists)
	{
		char hr[16],hx[16];
		sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO headers(hash,header,htext) "
									"VALUES(?,?,?);",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		if(article->refs.str)
		{
			md5_get_digest(article->refs.str,article->refs.len,hr);
			sqlite3_bind_blob(stmt,1,hr,16,SQLITE_STATIC);
			sqlite3_bind_text(stmt,2,HEADER_REFERENCES,-1,SQLITE_STATIC);
			sqlite3_bind_text(stmt,3,article->refs.str,article->refs.len,SQLITE_STATIC);
			sqlite3_step(stmt);
			sqlite3_reset(stmt);
		}
		if(article->xref.str)
		{
			md5_get_digest(article->xref.str,article->xref.len,hx);
			sqlite3_bind_blob(stmt,1,hx,16,SQLITE_STATIC);
			sqlite3_bind_text(stmt,2,HEADER_XREF,-1,SQLITE_STATIC);
			sqlite3_bind_text(stmt,3,article->xref.str,article->xref.len,SQLITE_STATIC);
			sqlite3_step(stmt);
		}
		sqlite3_finalize(stmt);
	
		sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO article_header(aid,hid,server_id) "
									"SELECT article.id,headers.id,server.id FROM article,server,headers "
									"WHERE article.id=?1 AND server.id=?2 AND headers.hash=?3;",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_int64(stmt,1,aid);
		sqlite3_bind_int64(stmt,2,sid);
		if(article->refs.str)
		{
			sqlite3_bind_blob(stmt,3,hr,16,SQLITE_STATIC);
			sqlite3_step(stmt);
			sqlite3_reset(stmt);
		}
		if(article->xref.str)
		{
			sqlite3_bind_blob(stmt,3,hx,16,SQLITE_STATIC);
			sqlite3_step(stmt);
		}
		sqlite3_finalize(stmt);
		
		if(base_id==0)
		{
			sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO refs(article_id,reference) "
										"VALUES(?,?);",-1,&stmt,NULL);
			SQL_PREP_VALID(sq_res,db);
			sqlite3_bind_int64(stmt,1,aid);
			GMimeReferences *refs,*refsi;
			char * ref=article->refs.str;
			if(ref && ref[0]!=0)
			{
				refs=refsi=g_mime_references_decode(ref);
				do
				{
					md5_get_digest(refsi->msgid,strlen(refsi->msgid),hash);
					sqlite3_bind_blob(stmt,2,hash,16,SQLITE_STATIC);
					sqlite3_step(stmt);
					sqlite3_reset(stmt);
					refsi=refsi->next;
				}while(refsi!=NULL);
				g_mime_references_clear(&refs);
			}
			sqlite3_finalize(stmt);
		}
	}

	if(!exists_base && base_id!=0)
	{
		// assume that all parts of a multi-part have the same refs
		sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO refs(article_id,reference) "
									"VALUES(?,?);",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_int64(stmt,1,base_id);
		GMimeReferences *refs,*refsi;
		char * ref=article->refs.str;
		if(ref && ref[0]!=0)
		{
			refs=refsi=g_mime_references_decode(ref);
			do
			{
				md5_get_digest(refsi->msgid,strlen(refsi->msgid),hash);
				sqlite3_bind_blob(stmt,2,hash,16,SQLITE_STATIC);
				sqlite3_step(stmt);
				sqlite3_reset(stmt);
				refsi=refsi->next;
			}while(refsi!=NULL);
			g_mime_references_clear(&refs);
		}
		sqlite3_finalize(stmt);
	}
	
	g_free(pre);
	g_free(norm);
	g_free(post);
	g_free(mid);
	internet_address_list_destroy(addr);
	
	if(parts>1)
	{
		Article *a=article_get_by_id(base_id);
		if(a!=NULL)
		{
			article_updt_multi(a);
			g_object_unref(a);
		}
	}
	debug_exit("add_article");
	
	return (parts>1)?base_id:aid;
}

/**
 * add_articles:
 * @gid: group id.
 * @sid: server id.
 * @articles: article_add_s **.
 * @qty: number of articles.
 * @status:
 *
 * Inserts multiple article overviews into the db.  It does this by wrapping 
 * add_article in a transaction.
 *
 * Returns: GArray of gint64 article ids.
 */
GArray* add_articles(gint32 gid, gint32 sid, const article_add_s **articles, int qty, StatusItem *status)
{
	g_return_val_if_fail(articles!=NULL,g_array_new(FALSE,FALSE,sizeof(gint64)));
	g_return_val_if_fail(qty>0,g_array_new(FALSE,FALSE,sizeof(gint64)));
	
	debug_enter("add_articles");
	
	gint32 i;
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res;
	char *errmsg;
	GArray *ret=g_array_sized_new(FALSE,TRUE,sizeof(gint64),qty);
	gint64 id;
	
	// FIXME: do something with status
	BEGIN_TRANSACTION(db,stmt);
	for(i=0;i<qty;i++)
	{
		id=add_article(gid,sid,articles[i]);
		g_array_append_val(ret,id);
	}
	END_TRANSACTION(db,stmt);
	
	debug_exit("add_articles");
	return ret;
}

/**
 * add_group:
 * @group:
 * @sid:
 *
 * Add a possibly new group to a server. No transaction is used.
 */
void add_group(const group_add_s *group, gint32 sid)
{
	g_assert(group!=NULL);
	g_assert(pstring_is_set(&group->name));
	g_assert(pstring_is_valid(&group->description));
	g_assert(group->name.len>0);
	g_assert(sid!=0);
	
	debug_enter("add_group");
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res,gid;
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO groups(name,description) "
								"VALUES(?,?);",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,group->name.str,group->name.len,SQLITE_STATIC);
	sqlite3_bind_text(stmt,2,group->description.str,group->description.len,SQLITE_STATIC);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	
	sq_res=sqlite3_prepare(db,"SELECT id FROM groups WHERE name=?;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,group->name.str,group->name.len,SQLITE_STATIC);
	sqlite3_step(stmt);
	gid=sqlite3_column_int(stmt,0);
	sqlite3_finalize(stmt);
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO group_server(group_id,server_id,permision) "
								"VALUES(?,?,?);",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_int(stmt,1,gid);
	sqlite3_bind_text(stmt,3,&group->permision,1,SQLITE_STATIC);
	sqlite3_bind_int(stmt,2,sid);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	
	debug_exit("add_group");
}

/**
 * add_groups:
 * @group:
 * @qty:
 * @sid:
 *
 * Add possibly new groups to the db. Uses add_group wrapped in a transaction.
 */
void add_groups(const group_add_s **groups, int qty, gint32 sid)
{
	g_return_if_fail(groups!=NULL);
	g_return_if_fail(qty>0);
	g_return_if_fail(sid!=0);
	
	debug_enter("add_groups");
	
	gint32 i;
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res;
	BEGIN_TRANSACTION(db,stmt);
	for(i=0;i<qty;i++)
		add_group(groups[i],sid);
	END_TRANSACTION(db,stmt);
	
	debug_exit("add_groups");
}

/**
 * remove_group:
 * &gid:
 * &sid:
 *
 * Removes a group from a server.
 */
void remove_group(gint32 gid, gint32 sid)
{
	g_assert(gid>0);
	g_assert(sid>0);
	
	debug_enter("remove_group");
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res;
	gint32 gsid=0;
	
	sq_res=sqlite3_prepare(db,"SELECT id FROM group_server WHERE group_id=? AND server_id=?;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_int(stmt,1,gid);
	sqlite3_bind_int(stmt,2,sid);
	if G_LIKELY(sqlite3_step(stmt)==SQLITE_ROW)
		gsid=sqlite3_column_int(stmt,0);
	sqlite3_finalize(stmt);
	
	if G_LIKELY(gsid>0)
	{
		group_do_expire_articles(gid,sid,0,0);
		
		sq_res=sqlite3_prepare(db,"DELETE FROM group_server WHERE id=?;",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_int(stmt,1,gsid);
		sqlite3_step(stmt);
		sqlite3_finalize(stmt);
		
		gboolean exists=FALSE;
		
		sq_res=sqlite3_prepare(db,"SELECT id FROM group_server WHERE group_id=? LIMIT 1;",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_int(stmt,1,gid);
		if(sqlite3_step(stmt)==SQLITE_ROW)
			exists=TRUE;
		sqlite3_finalize(stmt);
		
		if(!exists)
		{
			// group not attached to ANY server
			gint32 vsid;
			GPtrArray *grps=g_ptr_array_sized_new(1);
			
			g_ptr_array_add(grps,group_new(gid));
			
			vserver_remove_groups(1,grps);
			
			g_object_unref(GROUP(g_ptr_array_index(grps,0)));
			g_ptr_array_free(grps,TRUE);
		}
	}
	debug_exit("remove_group");
}

/**
 * sync_groups:
 * @groups: GPtrArray containing ALL group_add_s for a server.
 * @sid: server id.
 *
 * Syncs pans list of groups for a server with the list retreived from a server.
 * This function assumes @groups is the COMPLETE list of groups for the server.
 * It will remove any groups from the server that are NOT in the list.
 */
void sync_groups(GPtrArray *groups, gint32 sid)
{
	g_return_if_fail(groups!=NULL);
	g_return_if_fail(groups->len>0);
	g_return_if_fail(sid>0);
}

/**
 * add_group:
 * @group:
 *
 * Add a new folder to the db and the cache. No transaction is used.
 *
 * The group id or %0;
 */
gint32 add_folder(const group_add_s *group)
{
	g_assert(group!=NULL);
	g_assert(pstring_is_set(&group->name));
	g_assert(pstring_is_valid(&group->description));
	g_assert(group->name.len>0);
	
	debug_enter("add_folder");
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res,gid=0;
	
	sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO groups(name,description,folder,new) "
								"VALUES(?,?,1,0);",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_bind_text(stmt,1,group->name.str,group->name.len,SQLITE_STATIC);
	sqlite3_bind_text(stmt,2,group->description.str,group->description.len,SQLITE_STATIC);
	sqlite3_step(stmt);
	sqlite3_finalize(stmt);
	
	if(sqlite3_changes(db)==1)
	{
		gid=sqlite3_last_insert_rowid(db);
		
		sq_res=sqlite3_prepare(db,"INSERT OR IGNORE INTO group_server(group_id,server_id,permision) "
									"VALUES(?,0,'y');",-1,&stmt,NULL);
		SQL_PREP_VALID(sq_res,db);
		sqlite3_bind_int(stmt,1,gid);
		sqlite3_step(stmt);
		sqlite3_finalize(stmt);
		
		acache_add_folder(group->name.str);
	}
	
	debug_exit("add_folder");
	return gid;
}

