/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2004  K. Haley <haleykd@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DB_H
#define DB_H

#include <sqlite3.h>

/* retreive connection to database */
sqlite3* get_db(void);
void close_db(void);

/* construct database files if needed
** also migrates old data and folders
** returns true if files were craeted */
gboolean build_db(void);

/*
** verify that articles with refcnt==0
** are really gone.  Then delete them
** from article.
*/
void clean_db(void);

#define BEGIN_TRANSACTION(xdb,xstmt) \
{\
	sqlite3_prepare(xdb,"BEGIN TRANSACTION;",-1,&xstmt,NULL);\
	if(sqlite3_step(xstmt)!=SQLITE_DONE)\
	{\
		g_print("File: %s Line: %d\n%s\n",__FILE__,__LINE__,sqlite3_errmsg(xdb));\
	}\
	sqlite3_finalize(xstmt);\
}

#define END_TRANSACTION(xdb,xstmt) \
{\
	sqlite3_prepare(xdb,"COMMIT TRANSACTION;",-1,&xstmt,NULL);\
	if(sqlite3_step(xstmt)!=SQLITE_DONE)\
	{\
		g_print("File: %s Line: %d\n%s\n",__FILE__,__LINE__,sqlite3_errmsg(xdb));\
	}\
	sqlite3_finalize(xstmt);\
}

#define SQL_PREP_VALID(xresult,xdb) \
{\
	if G_UNLIKELY(xresult != SQLITE_OK)\
	{\
		g_print("sqlite3_prepare failed - File: %s Line: %d\n%s\n",__FILE__,__LINE__,sqlite3_errmsg(xdb));\
		g_error("sqlite3_prepare failed - File: %s Line: %d\n%s\n",__FILE__,__LINE__,sqlite3_errmsg(xdb));\
	}\
}\

#define SQL_EXEC_VALID(xresult,xmsg) \
{\
	if G_UNLIKELY(xresult != SQLITE_OK)\
	{\
		g_print("sqlite3_exec failed - File: %s Line: %d\n%s\n",__FILE__,__LINE__,xmsg);\
		g_error("sqlite3_exec failed - File: %s Line: %d\n%s\n",__FILE__,__LINE__,xmsg);\
		sqlite3_free(xmsg);\
	}\
}\

#endif
