/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2004  K. Haley <haleykd@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef VSERVER_H
#define VSERVER_H

#include <glib.h>
#include <pan/base/group.h>

typedef struct
{
	const char *name;
	guint32 id;
} vserver;

enum {
	VSERVER_ALL = 0,
	VSERVER_SUBSCRIBED,
	VSERVER_NEW,
	VSERVER_FOLDERS
};

/**
 * vserver_init:
 * 
 * Loads the list of vservers.
 *
 * Returns:
 **/
void vserver_init();

/**
 * vserver_shutdown:
 * 
 * Frees the list of vservers.
 *
 * Returns:
 **/
void vserver_shutdown();

/**
 * vserver_add:
 * @name: the name of the new vserver
 *
 * Adds a new vserver to the list. Fires the vserver_added signal on global_signals.
 *
 * Returns: TRUE if vserver was added, FALSE if the named vserver already exists.
 **/
gboolean vserver_add(const char *name);

/**
 * vserver_remove:
 * id: the id of the vserver to remove
 * 
 * Removes a vserver from the list and the database.  Fires vserver_removed on gloal_signals.
 *
 * Returns:
 **/
void vserver_remove(guint32 id);

/**
 * vserver_get_list:
 * 
 * Gets the list of vservers. ALL, SUBSCRIBED, NEW, and FOLDERS are always 
 * in the same positions in the list, and always have the same ids.
 *
 * Returns: GPtrArray of vserver structs.
 **/
const GPtrArray *vserver_get_list();

/**
 * vserver_get_groups:
 * id: id of the vserver
 * 
 * Loads the list of groups in the vservers.
 *
 * Returns: GPtrArray of Groups. Caller owns the array and the refed groups.
 **/
GPtrArray *vserver_get_groups(guint32 id);

/**
 * vserver_add_groups:
 * id: id of the vserver
 * groups: GPtrArray of Groups to add
 * 
 * Adds groups to the vserver.
 *
 * Returns: 
 **/
void vserver_add_groups(guint32 id, const GPtrArray *groups);

/**
 * vserver_remove_groups:
 * id: id of the vserver
 * groups: GPtrArray of groups to remove
 * 
 * Removes groups from the vserver.
 *
 * Returns: 
 **/
void vserver_remove_groups(guint32 id, const GPtrArray *groups);

#endif
