/* Generated by GOB (v2.0.12) on Fri Oct 07 00:43:20 2005
   (do not edit directly) */

/* End world hunger, donate to the World Food Programme, http://www.wfp.org */


#line 2 "pan/pan/socket-pool.gob"

/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2004  K. Haley <haleykd@users.sourceforge.net>
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#line 29 "socket-pool.c"
#define GOB_VERSION_MAJOR 2
#define GOB_VERSION_MINOR 0
#define GOB_VERSION_PATCHLEVEL 12

#define selfp (self->_priv)

#include <string.h> /* memset() */

#include "socket-pool.h"

#include "socket-pool-private.h"

#ifdef G_LIKELY
#define ___GOB_LIKELY(expr) G_LIKELY(expr)
#define ___GOB_UNLIKELY(expr) G_UNLIKELY(expr)
#else /* ! G_LIKELY */
#define ___GOB_LIKELY(expr) (expr)
#define ___GOB_UNLIKELY(expr) (expr)
#endif /* G_LIKELY */

#line 29 "pan/pan/socket-pool.gob"

#include <config.h>

#include <glib.h>
#include <glib/gi18n-lib.h>

#include <pan/base/debug.h>
#include <pan/base/log.h>

#include <pan/nntp.h>
#include <pan/queue.h>
#include <pan/sockets.h>
#include <pan/task.h>

#line 65 "socket-pool.c"

#line 44 "pan/pan/socket-pool.gob"

/*********************
**********************  Variables
*********************/

static GSList * pools;
static guint tag_upkeep = 0u;

static const guint KEEPALIVE_INTERVAL_SECS = 10u;
static const guint HUP_IDLE_TIME = 20u;

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/***
****  Channel Data
***/

typedef struct
{
	SocketPool * pool;
	gboolean is_checked_in;
	time_t last_active_time;
}
ChannelData;

static GHashTable *
channel_data_get_hash(void)
{
	static GHashTable * channel_data = NULL;

	if (channel_data == NULL)
		channel_data = g_hash_table_new_full(g_direct_hash,
		                                      g_direct_equal,
		                                      NULL,
		                                      g_free);

	return channel_data;
}

static ChannelData*
channel_data_get(GIOChannel * channel)
{
	GHashTable * hash = channel_data_get_hash();
	ChannelData * cd = (ChannelData*) g_hash_table_lookup(hash, channel);
	if (cd == NULL)
	{
		cd = g_new0(ChannelData, 1);
		cd->pool = NULL;
		cd->is_checked_in = TRUE;
		cd->last_active_time = (time_t)0;
		g_hash_table_insert(hash, channel, cd);
	}
	return cd;
}

static void
channel_data_free(GIOChannel * channel)
{
	g_hash_table_remove(channel_data_get_hash(), channel);
}


/***
****
****  Life Cycle
****
***/

static void
channel_upkeep_done(GIOChannel *channel, TaskStateEnum val, gpointer unused)
{
	socket_pool_checkin(channel, val==TASK_OK);
}
static void
channel_upkeep(gpointer channel_gpointer, gpointer unused)
{
	GIOChannel * channel = (GIOChannel*) channel_gpointer;
	ChannelData * cd = channel_data_get(channel);
	const int idle_secs = (int)difftime(time(NULL),cd->last_active_time);

	/* send keepalives for awhile, then hang up. */
	if (idle_secs > HUP_IDLE_TIME)
		socket_pool_checkin(channel, FALSE);
	else 
		nntp_noop(channel, channel_upkeep_done, NULL);
}
static void
pool_upkeep(gpointer pool_gpointer, gpointer unused)
{
	SocketPool * self = (SocketPool*) pool_gpointer;
	GSList * channels;

	/* keepalive each channel */
	channels = selfp->checked_in;
	selfp->checked_in = NULL;
	g_slist_foreach(channels, channel_upkeep, NULL);
	g_slist_free(channels);
}
static gboolean
pools_upkeep_timer(gpointer unused)
{
	g_slist_foreach(pools, pool_upkeep, NULL);
	return TRUE; /* upkeep every KEEPALIVE_INTERVAL_SECS */
}


#line 176 "socket-pool.c"

#line 153 "pan/pan/socket-pool.gob"

static void
add_channel_to_pool(SocketPool   * self,
                     GIOChannel   * channel)
{
	ChannelData * cd = channel_data_get(channel);
	cd->pool = self;
	cd->is_checked_in = TRUE;
	selfp->checked_in = g_slist_append(selfp->checked_in, channel);
}

static void
handshake_done_func(GIOChannel       * channel,
                     TaskStateEnum      state,
                     gpointer           user_data)
{
	SocketPool * self = (SocketPool*) user_data;

	if (state != TASK_OK)
	{
		channel_data_free(channel);
		g_io_channel_shutdown(channel, FALSE, NULL);
		g_io_channel_unref(channel);
		channel = NULL;
	}

	if (state == TASK_FAIL)  /* the news server doesn't love us anymore! */
	{
		queue_set_online(FALSE);
	}

	/* if success ... */
	if (channel != NULL)
	{
		/* update the pool */
		debug2(DEBUG_QUEUE, "success with handshake %s, channel %p",
		                     selfp->server->name,
		                     channel);

		selfp->connection_pending = FALSE;
		++selfp->socket_qty;
		channel_data_get(channel)->last_active_time = time(NULL);
		add_channel_to_pool(self, channel);

		/* let clients know */
		socket_pool_connection_available_changed(self,selfp->server);
	}
}

#line 228 "socket-pool.c"

#line 203 "pan/pan/socket-pool.gob"

static void
goodbye_done(GIOChannel      * channel,
              TaskStateEnum     ignored,
              gpointer          user_data)
{
	SocketPool * self = (SocketPool*) user_data;	
	debug_enter("goodbye_done");
	/* note that we don't check the outcome of the command --
	   if `QUIT' has failed there's nothing we can do anyway. */
	channel_data_free(channel);
	g_io_channel_shutdown(channel, FALSE, NULL);
	g_io_channel_unref(channel);
	--selfp->socket_qty;
	socket_pool_connection_available_changed(self,selfp->server);
	debug_exit("goodbye_done");
}


#line 250 "socket-pool.c"
/* self casting macros */
#define SELF(x) SOCKET_POOL(x)
#define SELF_CONST(x) SOCKET_POOL_CONST(x)
#define IS_SELF(x) IS_SOCKET_POOL(x)
#define TYPE_SELF TYPE_SOCKET_POOL
#define SELF_CLASS(x) SOCKET_POOL_CLASS(x)

#define SELF_GET_CLASS(x) SOCKET_POOL_GET_CLASS(x)

/* self typedefs */
typedef SocketPool Self;
typedef SocketPoolClass SelfClass;

/* here are local prototypes */
static void socket_pool_init (SocketPool * o) G_GNUC_UNUSED;
static void socket_pool_class_init (SocketPoolClass * c) G_GNUC_UNUSED;

/*
 * Signal connection wrapper macro shortcuts
 */
#define self_connect__connection_available_changed(object,func,data)	socket_pool_connect__connection_available_changed((object),(func),(data))
#define self_connect_after__connection_available_changed(object,func,data)	socket_pool_connect_after__connection_available_changed((object),(func),(data))
#define self_connect_data__connection_available_changed(object,func,data,destroy_data,flags)	socket_pool_connect_data__connection_available_changed((object),(func),(data),(destroy_data),(flags))

typedef void  (*___Sig1) (SocketPool *, GObject *, gpointer);

static void
___marshal_Sig1 (GClosure *closure,
	GValue *return_value G_GNUC_UNUSED,
	guint n_param_values,
	const GValue *param_values,
	gpointer invocation_hint G_GNUC_UNUSED,
	gpointer marshal_data)
{
	register ___Sig1 callback;
	register GCClosure *cc = (GCClosure*) closure;
	register gpointer data1, data2;

	g_return_if_fail (n_param_values == 2);

	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = g_value_peek_pointer (param_values + 0);
	} else {
		data1 = g_value_peek_pointer (param_values + 0);
		data2 = closure->data;
	}

	callback = (___Sig1) (marshal_data != NULL ? marshal_data : cc->callback);

	callback ((SocketPool *)data1,
		(GObject *) g_value_get_object (param_values + 1),
		data2);
}


enum {
	CONNECTION_AVAILABLE_CHANGED_SIGNAL,
	LAST_SIGNAL
};

static guint object_signals[LAST_SIGNAL] = {0};

/* pointer to the class of our parent */
static GObjectClass *parent_class = NULL;

/* Short form macros */
#define self_connection_available_changed socket_pool_connection_available_changed
#define self_new socket_pool_new
#define self_free socket_pool_free
#define self_request_connection socket_pool_request_connection
#define self_checkout socket_pool_checkout
#define self_checkin socket_pool_checkin
#define self_get_connection_qty socket_pool_get_connection_qty
#define self_set_online socket_pool_set_online
GType
socket_pool_get_type (void)
{
	static GType type = 0;

	if ___GOB_UNLIKELY(type == 0) {
		static const GTypeInfo info = {
			sizeof (SocketPoolClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) socket_pool_class_init,
			(GClassFinalizeFunc) NULL,
			NULL /* class_data */,
			sizeof (SocketPool),
			8 /* n_preallocs */,
			(GInstanceInitFunc) socket_pool_init,
			NULL
		};

		type = g_type_register_static (G_TYPE_OBJECT, "SocketPool", &info, (GTypeFlags)0);
	}

	return type;
}

/* a macro for creating a new object of our type */
#define GET_NEW ((SocketPool *)g_object_new(socket_pool_get_type(), NULL))

/* a function for creating a new object of our type */
#include <stdarg.h>
static SocketPool * GET_NEW_VARG (const char *first, ...) G_GNUC_UNUSED;
static SocketPool *
GET_NEW_VARG (const char *first, ...)
{
	SocketPool *ret;
	va_list ap;
	va_start (ap, first);
	ret = (SocketPool *)g_object_new_valist (socket_pool_get_type (), first, ap);
	va_end (ap);
	return ret;
}


static void
___finalize(GObject *obj_self)
{
#define __GOB_FUNCTION__ ":Socket:Pool::finalize"
	SocketPool *self G_GNUC_UNUSED = SOCKET_POOL (obj_self);
	gpointer priv G_GNUC_UNUSED = self->_priv;
	if(G_OBJECT_CLASS(parent_class)->finalize) \
		(* G_OBJECT_CLASS(parent_class)->finalize)(obj_self);
#line 230 "pan/pan/socket-pool.gob"
	if(self->_priv->checked_in) { g_slist_free ((gpointer) self->_priv->checked_in); self->_priv->checked_in = NULL; }
#line 379 "socket-pool.c"
}
#undef __GOB_FUNCTION__

static void 
socket_pool_init (SocketPool * o G_GNUC_UNUSED)
{
#define __GOB_FUNCTION__ ":Socket:Pool::init"
	o->_priv = G_TYPE_INSTANCE_GET_PRIVATE(o,TYPE_SELF,SocketPoolPrivate);
#line 203 "pan/pan/socket-pool.gob"
	o->_priv->connection_pending = FALSE;
#line 390 "socket-pool.c"
#line 203 "pan/pan/socket-pool.gob"
	o->_priv->online = TRUE;
#line 393 "socket-pool.c"
#line 203 "pan/pan/socket-pool.gob"
	o->_priv->socket_qty = 0;
#line 396 "socket-pool.c"
#line 203 "pan/pan/socket-pool.gob"
	o->_priv->server = NULL;
#line 399 "socket-pool.c"
#line 203 "pan/pan/socket-pool.gob"
	o->_priv->checked_in = NULL;
#line 402 "socket-pool.c"
}
#undef __GOB_FUNCTION__
static void 
socket_pool_class_init (SocketPoolClass * c G_GNUC_UNUSED)
{
#define __GOB_FUNCTION__ ":Socket:Pool::class_init"
	GObjectClass *g_object_class G_GNUC_UNUSED = (GObjectClass*) c;

	g_type_class_add_private(c,sizeof(SocketPoolPrivate));

	parent_class = g_type_class_ref (G_TYPE_OBJECT);

	object_signals[CONNECTION_AVAILABLE_CHANGED_SIGNAL] =
		g_signal_new ("connection_available_changed",
			G_TYPE_FROM_CLASS (g_object_class),
			(GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
			G_STRUCT_OFFSET (SocketPoolClass, connection_available_changed),
			NULL, NULL,
			___marshal_Sig1,
			G_TYPE_NONE, 1,
			G_TYPE_OBJECT);
	if ___GOB_UNLIKELY(sizeof(Server * ) != sizeof(GObject *) || parent_class == NULL /* avoid warning */) {
		g_error("pan/pan/socket-pool.gob line 232: Type mismatch of \"connection_available_changed\" signal signature");
	}

	c->connection_available_changed = NULL;
	g_object_class->finalize = ___finalize;
}
#undef __GOB_FUNCTION__



#line 232 "pan/pan/socket-pool.gob"
void 
socket_pool_connection_available_changed (SocketPool * self, Server * server)
#line 438 "socket-pool.c"
{
	GValue ___param_values[2];
	GValue ___return_val;

memset (&___return_val, 0, sizeof (___return_val));
memset (&___param_values, 0, sizeof (___param_values));

#line 232 "pan/pan/socket-pool.gob"
	g_return_if_fail (self != NULL);
#line 232 "pan/pan/socket-pool.gob"
	g_return_if_fail (IS_SOCKET_POOL (self));
#line 450 "socket-pool.c"

	___param_values[0].g_type = 0;
	g_value_init (&___param_values[0], G_TYPE_FROM_INSTANCE (self));
	g_value_set_instance (&___param_values[0], (gpointer) self);

	___param_values[1].g_type = 0;
	g_value_init (&___param_values[1], G_TYPE_OBJECT);
	g_value_set_object (&___param_values[1], (GObject *) server);

	g_signal_emitv (___param_values,
		object_signals[CONNECTION_AVAILABLE_CHANGED_SIGNAL],
		0 /* detail */,
		&___return_val);

	g_value_unset (&___param_values[0]);
	g_value_unset (&___param_values[1]);
}

/**
 * socket_pool_new:
 * @server:
 *
 * new socket pool.
 *
 * Returns:
 **/
#line 243 "pan/pan/socket-pool.gob"
SocketPool * 
socket_pool_new (Server * server)
#line 480 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::new"
#line 243 "pan/pan/socket-pool.gob"
	g_return_val_if_fail (server != NULL, (SocketPool * )0);
#line 243 "pan/pan/socket-pool.gob"
	g_return_val_if_fail (IS_SERVER (server), (SocketPool * )0);
#line 487 "socket-pool.c"
{
#line 244 "pan/pan/socket-pool.gob"
	
		Self *self=GET_NEW;
		
		selfp->server=server;
		/* static class upkeep */
		pools = g_slist_prepend(pools, self);
		if (!tag_upkeep)
		{
			const int msec = KEEPALIVE_INTERVAL_SECS * 1000;
			tag_upkeep = g_timeout_add(msec, pools_upkeep_timer, NULL);
		}
		
		return self;
	}}
#line 504 "socket-pool.c"
#undef __GOB_FUNCTION__

/**
 * socket_pool_free:
 * @self:
 *
 * not quite an Alias for g_object_unref so I don't have as much to re-write.
 **/
#line 265 "pan/pan/socket-pool.gob"
void 
socket_pool_free (SocketPool * self)
#line 516 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::free"
#line 265 "pan/pan/socket-pool.gob"
	g_return_if_fail (self != NULL);
#line 265 "pan/pan/socket-pool.gob"
	g_return_if_fail (IS_SOCKET_POOL (self));
#line 523 "socket-pool.c"
{
#line 266 "pan/pan/socket-pool.gob"
	
		pools=g_slist_remove(pools,self);
		
		g_object_unref(self);
	}}
#line 531 "socket-pool.c"
#undef __GOB_FUNCTION__

#line 272 "pan/pan/socket-pool.gob"
void 
socket_pool_request_connection (SocketPool * self)
#line 537 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::request_connection"
#line 272 "pan/pan/socket-pool.gob"
	g_return_if_fail (self != NULL);
#line 272 "pan/pan/socket-pool.gob"
	g_return_if_fail (IS_SOCKET_POOL (self));
#line 544 "socket-pool.c"
{
#line 273 "pan/pan/socket-pool.gob"
	
		debug_enter("socket_pool_request_connection");
		/* If we're online,
		 * and we don't have any sockets handy,
		 * and there aren't any pending connections,
		 * and we haven't maxed out the number of connections,
		 * then try to make a new connection. */
		if ((selfp->online) &&
			(selfp->checked_in == NULL) &&
			(selfp->connection_pending == FALSE) &&
			(selfp->server->max_connections - selfp->socket_qty > 0))
		{
			GIOChannel * channel;
			Server * server = selfp->server;
	
			/* create the socket */
			debug1(DEBUG_QUEUE, "connecting to server %s", server->name);
			channel = pan_get_giochannel_socket(&server->address, server->port);
			if (channel != NULL)
			{
				selfp->connection_pending = TRUE;
	
				nntp_handshake(channel,
								server->username.str,
								server->password.str,
								handshake_done_func,
								self);
			}
		}
		debug_exit("socket_pool_request_connection");
	}}
#line 578 "socket-pool.c"
#undef __GOB_FUNCTION__

#line 305 "pan/pan/socket-pool.gob"
GIOChannel * 
socket_pool_checkout (SocketPool * self)
#line 584 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::checkout"
#line 305 "pan/pan/socket-pool.gob"
	g_return_val_if_fail (self != NULL, (GIOChannel * )0);
#line 305 "pan/pan/socket-pool.gob"
	g_return_val_if_fail (IS_SOCKET_POOL (self), (GIOChannel * )0);
#line 591 "socket-pool.c"
{
#line 306 "pan/pan/socket-pool.gob"
	
		GIOChannel * channel = NULL;
		
		debug_enter("socket_pool_checkout");
		/* pop the first channel from the checked_in list */
		if (selfp->online && selfp->checked_in!=NULL)
		{
			GSList * pop = selfp->checked_in;
			selfp->checked_in = g_slist_remove_link(selfp->checked_in, pop);
			channel = (GIOChannel*) pop->data;
			g_slist_free_1(pop);
		}
	
		/* if we got a channel, mark it as `checked out' */
		if (channel != NULL)
		{
			ChannelData * cd = channel_data_get(channel);
			cd->is_checked_in = FALSE;
			debug2(DEBUG_QUEUE, "checkout pool %p channel %p", self, channel);
		}
		
		debug_exit("socket_pool_checkout");
		return channel;
	}}
#line 618 "socket-pool.c"
#undef __GOB_FUNCTION__

#line 331 "pan/pan/socket-pool.gob"
void 
socket_pool_checkin (GIOChannel * channel, gboolean channel_ok)
#line 624 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::checkin"
#line 331 "pan/pan/socket-pool.gob"
	g_return_if_fail (channel != NULL);
#line 629 "socket-pool.c"
{
#line 332 "pan/pan/socket-pool.gob"
	
		ChannelData * cd;
		Self * self;
	
		cd = channel_data_get(channel);
		self = cd->pool;
		debug2(DEBUG_QUEUE, "checkin pool %p channel %p", self, channel);
	
		if (!cd->is_checked_in)
			cd->last_active_time = time(NULL);
	
		if (channel_ok
			&& selfp->socket_qty <= selfp->server->max_connections
			&& selfp->online)
		{
			debug0(DEBUG_QUEUE, "adding back to list");
			add_channel_to_pool(self, channel);
		}
		else
		{
			debug0(DEBUG_QUEUE, "error or offline or too many connections -- closing");
			nntp_goodbye(channel, goodbye_done, self);
		}
	}}
#line 656 "socket-pool.c"
#undef __GOB_FUNCTION__

#line 357 "pan/pan/socket-pool.gob"
int 
socket_pool_get_connection_qty (SocketPool const * self)
#line 662 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::get_connection_qty"
#line 357 "pan/pan/socket-pool.gob"
	g_return_val_if_fail (self != NULL, (int )0);
#line 357 "pan/pan/socket-pool.gob"
	g_return_val_if_fail (IS_SOCKET_POOL (self), (int )0);
#line 669 "socket-pool.c"
{
#line 358 "pan/pan/socket-pool.gob"
	
		return selfp->socket_qty;
	}}
#line 675 "socket-pool.c"
#undef __GOB_FUNCTION__

/**
 * socket_pool_set_online:
 * @self:
 * @online:
 *
 * If online is set to false, requests will be ignored,
 * checkout will always return NULL, and all checked-in connections
 * will be closed when refresh() is called.
 **/
#line 371 "pan/pan/socket-pool.gob"
void 
socket_pool_set_online (SocketPool * self, gboolean online)
#line 690 "socket-pool.c"
{
#define __GOB_FUNCTION__ ":Socket:Pool::set_online"
#line 371 "pan/pan/socket-pool.gob"
	g_return_if_fail (self != NULL);
#line 371 "pan/pan/socket-pool.gob"
	g_return_if_fail (IS_SOCKET_POOL (self));
#line 697 "socket-pool.c"
{
#line 372 "pan/pan/socket-pool.gob"
	
		if (online != selfp->online)
		{
			selfp->online = online;
			
			self_connection_available_changed(self,selfp->server);
		}
	}}
#line 708 "socket-pool.c"
#undef __GOB_FUNCTION__
