#include <glib.h>
#include <gtk/gtk.h>

#include <pan/base/group.h>
#include <pan/base/vserver.h>
#include <pan/base/base-prefs.h>

static GtkWidget *window = NULL;


enum
{
  COLUMN_NAME,
  COLUMN_UNREAD,
  COLUMN_TOTAL,
  COLUMN_DESC,
  COLUMN_OBJECT,
  NUM_COLUMNS
};

static int iter_sort(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
{
	Group *ga, *gb;
	gtk_tree_model_get(model,a,COLUMN_OBJECT,&ga,-1);
	gtk_tree_model_get(model,b,COLUMN_OBJECT,&gb,-1);
	
	switch(GPOINTER_TO_INT(data))
	{
		case COLUMN_NAME: return pstring_compare(group_get_name(ga),group_get_name(gb));
		case COLUMN_TOTAL:
			if(ga->article_qty==gb->article_qty) return 0;
			if(ga->article_qty<gb->article_qty) return -1;
			return 1;
		case COLUMN_UNREAD:
		{
			int ua=ga->article_qty-ga->article_read_qty;
			int ub=gb->article_qty-gb->article_read_qty;
			if(ua==ub) return 0;
			if(ua<ub) return -1;
			return 1;
		}
	}
	return 0;
}

static GtkTreeModel *
create_model (void)
{
  gint i = 0;
  gulong x;
  GtkListStore *store;
  GtkTreeIter iter;
  GTimer *timer=g_timer_new();
  g_timer_start(timer);
  GPtrArray *a=vserver_get_groups(VSERVER_ALL);
  g_timer_stop(timer);
  g_message("loaded %d group in %f seconds.",a->len,g_timer_elapsed(timer,&x));
  Group *g;
  char buf[10];
  const char *name, *desc;

  /* create list store */
  store = gtk_list_store_new (NUM_COLUMNS,
  					G_TYPE_STRING,
					G_TYPE_ULONG,
					G_TYPE_ULONG,
					G_TYPE_STRING,
				  G_TYPE_OBJECT
				  );
  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store),COLUMN_NAME,iter_sort,GINT_TO_POINTER(COLUMN_NAME),NULL);
  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store),COLUMN_UNREAD,iter_sort,GINT_TO_POINTER(COLUMN_UNREAD),NULL);
  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store),COLUMN_TOTAL,iter_sort,GINT_TO_POINTER(COLUMN_TOTAL),NULL);

  /* add data to the list store */
  g_timer_reset(timer);
  g_timer_start(timer);
  for (i = 0; i < a->len; i++)
    {
		g=GROUP(g_ptr_array_index(a,i));
      gtk_list_store_append (store, &iter);
      gtk_list_store_set (store, &iter,
			  COLUMN_OBJECT,g,
			  COLUMN_TOTAL, g->article_qty,
			  -1);
		g_object_unref(g);
    }
	g_ptr_array_free(a,TRUE);
	g_timer_stop(timer);
	  g_message("added groups in %f seconds.",g_timer_elapsed(timer,&x));
	 g_timer_destroy(timer);
  return GTK_TREE_MODEL (store);
}
/**
*** cell data functions
**/
static void group_name_col(GtkTreeViewColumn *col, GtkCellRenderer *cell, 
	GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	Group *g;
	gtk_tree_model_get(model,iter,COLUMN_OBJECT,&g,-1);
	const gchar *name=group_get_name(g)->str;
	int weight=(g->article_qty-g->article_read_qty)>0?PANGO_WEIGHT_BOLD:PANGO_WEIGHT_NORMAL;
	g_object_set(cell,"text",name,"weight",weight,NULL);
}
static void group_desc_col(GtkTreeViewColumn *col, GtkCellRenderer *cell, GtkTreeModel *model,
					GtkTreeIter *iter, gpointer data)
{
	Group *g;
	gtk_tree_model_get(model,iter,COLUMN_OBJECT,&g,-1);
	const gchar *name=group_get_description(g);
	
	g_object_set(cell,"text",name,NULL);
}
static void group_unread_col(GtkTreeViewColumn *col, GtkCellRenderer *cell, GtkTreeModel *model,
					GtkTreeIter *iter, gpointer data)
{
	Group *g;
	gtk_tree_model_get(model,iter,COLUMN_OBJECT,&g,-1);
	gint cnt=g->article_qty-g->article_read_qty;
	int weight=cnt>0?PANGO_WEIGHT_BOLD:PANGO_WEIGHT_NORMAL;
	gchar buf[12];
	
	g_snprintf(buf,12,"%d",cnt);
	g_object_set(cell,"text",buf,"weight",weight,NULL);
}
static void group_total_col(GtkTreeViewColumn *col, GtkCellRenderer *cell, GtkTreeModel *model,
					GtkTreeIter *iter, gpointer data)
{
	Group *g;
	gtk_tree_model_get(model,iter,COLUMN_OBJECT,&g,-1);
	gint cnt=g->article_qty;
	int weight=(cnt-g->article_read_qty)>0?PANGO_WEIGHT_BOLD:PANGO_WEIGHT_NORMAL;
	gchar buf[12];
	
	g_snprintf(buf,12,"%d",cnt);
	g_object_set(cell,"text",buf,"weight",weight,NULL);
}

static void
add_columns (GtkTreeView *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkTreeModel *model = gtk_tree_view_get_model (treeview);

  /* column for bug numbers */
  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new();
  //column=gtk_tree_view_column_new_with_attributes("Name",renderer,NULL);
  gtk_tree_view_column_set_title(column,"NAME");
  gtk_tree_view_column_pack_start(column,renderer,TRUE);
  gtk_tree_view_column_set_resizable(column,TRUE);
  gtk_tree_view_column_set_sizing(column,GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column,200);
  gtk_tree_view_column_set_cell_data_func(column,renderer,group_name_col,NULL,NULL);
  gtk_tree_view_append_column(treeview, column);
  gtk_tree_view_column_set_sort_column_id(column, COLUMN_NAME);
  
  renderer = gtk_cell_renderer_text_new ();
//  column = gtk_tree_view_column_new();
  column=gtk_tree_view_column_new_with_attributes("Unread",renderer,NULL);
    //gtk_tree_view_column_set_title(column,"Unread");
	gtk_tree_view_column_set_cell_data_func(column,renderer,group_unread_col,NULL,NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_UNREAD);
  gtk_tree_view_append_column (treeview, column);
  gtk_tree_view_column_set_sizing(column,GTK_TREE_VIEW_COLUMN_GROW_ONLY);

  renderer = gtk_cell_renderer_text_new ();
  column=gtk_tree_view_column_new_with_attributes("Total",renderer,NULL);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TOTAL);
  gtk_tree_view_append_column (treeview, column);
  gtk_tree_view_column_set_sizing(column,GTK_TREE_VIEW_COLUMN_GROW_ONLY);
  gtk_tree_view_column_set_cell_data_func(column,renderer,group_total_col,NULL,NULL);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new();
  gtk_tree_view_column_pack_start(column,renderer,TRUE);
  gtk_tree_view_column_set_title(column,"Description");
  gtk_tree_view_column_set_cell_data_func(column,renderer,group_desc_col,NULL,NULL);
  gtk_tree_view_append_column (treeview, column);
  gtk_tree_view_column_set_resizable(column,TRUE);
  gtk_tree_view_column_set_sizing(column,GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column,200);
}

GtkWidget *
do_list_store (GtkWidget *do_widget)
{
  if (!window)
    {
      GtkWidget *vbox;
      GtkWidget *label;
      GtkWidget *sw;
      GtkTreeModel *model;
      GtkWidget *treeview;

      /* create window, etc */
      window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
      gtk_window_set_title (GTK_WINDOW (window), "GtkListStore demo");

      g_signal_connect (window, "destroy",
			G_CALLBACK (gtk_widget_destroyed), &window);
      gtk_container_set_border_width (GTK_CONTAINER (window), 8);

      vbox = gtk_vbox_new (FALSE, 8);
      gtk_container_add (GTK_CONTAINER (window), vbox);

      label = gtk_label_new ("Group list.");
      gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

      sw = gtk_scrolled_window_new (NULL, NULL);
      gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
					   GTK_SHADOW_ETCHED_IN);
      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				      GTK_POLICY_NEVER,
				      GTK_POLICY_AUTOMATIC);
      gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);

      /* create tree view */
      treeview = gtk_tree_view_new();
      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
      /* add columns to the tree view */
      add_columns (GTK_TREE_VIEW (treeview));

      /* create tree model */
      model = create_model ();
	  gtk_tree_view_set_model(GTK_TREE_VIEW(treeview),model);
      g_object_unref (model);

      gtk_container_add (GTK_CONTAINER (sw), treeview);

      /* finish & show */
      gtk_window_set_default_size (GTK_WINDOW (window), 280, 250);
    }

  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show_all (window);
  else
    {
      gtk_widget_destroy (window);
      window = NULL;
    }

  return window;
}

static gboolean delete_event( GtkWidget *widget,
                              GdkEvent  *event,
                              gpointer   data )
{
    /* If you return FALSE in the "delete_event" signal handler,
     * GTK will emit the "destroy" signal. Returning TRUE means
     * you don't want the window to be destroyed.
     * This is useful for popping up 'are you sure you want to quit?'
     * type dialogs. */

    g_print ("delete event occurred\n");

    /* Change TRUE to FALSE and the main window will be destroyed with
     * a "delete_event". */

    return FALSE;
}
static void destroy( GtkWidget *widget,
                     gpointer   data )
{
    gtk_main_quit ();
}
int main( int   argc,
          char *argv[] )
{
    gtk_init (&argc, &argv);
 	base_prefs_init (g_get_tmp_dir(), 100, FALSE, FALSE);
   
    window = do_list_store(NULL);
	g_signal_connect (G_OBJECT (window), "delete_event",
		      G_CALLBACK (delete_event), NULL);
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (destroy), NULL);
			  
	gtk_widget_show  (window);
    
    gtk_main ();
    
    return 0;
}

