#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>


#include <pan/base/base-prefs.h>
#include <pan/base/util-file.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/vserver.h>
#include <pan/base/group.h>
#include <pan/base/group-private.h>
#include <pan/base/article.h>
#include <pan/base/db.h>
#include <pan/base/db-add.h>
#include <gmime/gmime.h>
#include <pan/base/debug.h>
#include <sqlite3.h>

static void init_pan(void)
{
 	char * base;
	char * data_dir;
	char *fn;

	g_mime_init(0);
	base = getenv ("srcdir");
	base = g_strdup_printf ("%s", base!=NULL ? base : getenv("PWD"));
	fn=g_strdup_printf("%s/data/pan.sqlite3",base);
	unlink(fn);
	data_dir = g_strdup_printf ("PAN_HOME=%s/data/", base);
	//g_print("data_dir= %s\n",data_dir);
	putenv (data_dir);
	base_prefs_init (g_get_tmp_dir(), 100, FALSE, FALSE);

	g_free(fn);
	g_free (data_dir);
	g_free (base);
}


static Server* get_server(void)
{
	const char *name="test-article-serveer";
	PString pname=PSTRING_INIT;
	pname=pstring_shallow(name,strlen(name));
	Server *s;
	s=serverlist_get_named_server(&pname);
	if(!s)
	{
		GError *err=NULL;
		s=server_new();
		pstring_copy(&s->name,&pname);
		pstring_set(&s->address,"foobar",6);
		serverlist_add_server(s,&err);
		if(err)
		{
			g_message("Error: %s",err->message);
			g_error_free(err);
		}
	}
	return s;
}

static GPtrArray* do_add_groups(void)
{
	int i;
	const char *names[]={"group 1","group 2","group 3"};
	const char *descs[]={"d1","no bodys home",""};
	const int len=3;
	group_add_s *gas;
	GPtrArray *grps=g_ptr_array_sized_new(len);
	Group *group;
	Server *server;
	
	server=get_server();
	
	for(i=0;i<len;i++)
	{
		gas=g_new0(group_add_s,1);
		gas->name=pstring_shallow(names[i],strlen(names[i]));
		gas->description=pstring_shallow(descs[i],strlen(descs[i]));
		gas->permision='n';
		g_ptr_array_add(grps,gas);
	}
	add_groups((const group_add_s**)grps->pdata,len,server->id);
	for(i=0;i<len;i++)
	{
		gas=g_ptr_array_index(grps,i);
		g_free(gas);
	}
	g_ptr_array_free(grps,TRUE);
	grps=vserver_get_groups(2); //get NEW groups
	if(grps->len!=3)
	{
		g_message("Error: expected 3 groups got %d",grps->len);
	}

	return grps;
}

static GPtrArray* make_articles(void)
{
	GPtrArray *ret=g_ptr_array_sized_new(3);
	article_add_s *a=NULL;
	
	a=g_new0(article_add_s,1);
	a->uid=1;
	a->date=25;
	a->byte_qty=10;
	a->lines=1;
	a->subject=pstring_shallow("This is article 1",-1);
	a->author=pstring_shallow("hello kitty <hkitty@no.server>",-1);
	a->msg_id=pstring_shallow("<12345@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);

	a=g_new0(article_add_s,1);
	a->uid=2;
	a->date=27;
	a->byte_qty=100;
	a->lines=10;
	a->subject=pstring_shallow("RE: This is article 1",-1);
	a->author=pstring_shallow("\"hello kitty\" <h.kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<21345@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);

	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("This is article 3",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31425@no>",-1);
	a->refs=pstring_shallow("<12345@no>",-1);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	
	return ret;
}

static int test_article(void)
{
	int ret=0,i,rc;
	GPtrArray *grps,*arts,*tarts;
	Server *server;
	Group *group;
	Article *a;
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res,old_cnt;
	
	//set_debug_level(DEBUG_TRACE);
	serverlist_init();
	vserver_init();
	server=get_server();
	grps=do_add_groups();
	arts=make_articles();
	
	group=g_ptr_array_index(grps,0);
	add_articles(group->id,server->id,(const article_add_s**)arts->pdata,arts->len);
	tarts=group_get_all_articles(group,NULL);
	if(tarts->len!=arts->len)
	{
		g_message("FAILED: adding articles expected 3 got %d",tarts->len);
		return 1;
	}
	
	sq_res=sqlite3_prepare(db,"SELECT count(refs.id) FROM article,refs WHERE date=35 AND refs.article_id=article.id;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_step(stmt);
	old_cnt=sqlite3_column_int(stmt,0);
	sqlite3_finalize(stmt);
	
	group=g_ptr_array_index(grps,1);
	add_article(group->id,server->id,(article_add_s*)arts->pdata[0]);
	add_article(group->id,server->id,(article_add_s*)arts->pdata[2]);
	group=g_ptr_array_index(grps,2);
	add_article(group->id,server->id,(article_add_s*)arts->pdata[0]);
	add_article(group->id,server->id,(article_add_s*)arts->pdata[1]);
	for(i=0;i<arts->len;i++)
		g_free(g_ptr_array_index(arts,i));
	g_ptr_array_free(arts,TRUE);

	sq_res=sqlite3_prepare(db,"SELECT count(refs.id) FROM article,refs WHERE date=35 AND refs.article_id=article.id;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_step(stmt);
	if(old_cnt!=sqlite3_column_int(stmt,0))
	{
		++ret;
		g_message("ERROR: Multiple insert refs count wrong. expected %d got %d",old_cnt,sqlite3_column_int(stmt,0));
	}else
		g_message("PASSED: Multiple insert.");
	sqlite3_finalize(stmt);
	
	group=g_ptr_array_index(grps,0);
	
	sq_res=sqlite3_prepare(db,"SELECT refcnt FROM article WHERE id=?;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	a=ARTICLE(g_ptr_array_index(tarts,0));
	article_set_old(a);
	sqlite3_bind_int64(stmt,1,a->id);
	sqlite3_step(stmt);
	rc=sqlite3_column_int(stmt,0);
	if(a->date!=35 || rc!=2)
	{
		++ret;
		g_message("ERROR: expected 35,2 got %d,%d",a->date,rc);
	}
	g_object_unref(a);
	a=ARTICLE(g_ptr_array_index(tarts,1));
	article_set_read(a,TRUE);
	sqlite3_reset(stmt);
	sqlite3_bind_int64(stmt,1,a->id);
	sqlite3_step(stmt);
	rc=sqlite3_column_int(stmt,0);
	if(a->date!=27 || rc!=2)
	{
		++ret;
		g_message("ERROR: expected 27,2 got %d,%d",a->date,rc);
	}
	g_object_unref(a);
	a=ARTICLE(g_ptr_array_index(tarts,2));
	article_set_flagged(a,TRUE);
	sqlite3_reset(stmt);
	sqlite3_bind_int64(stmt,1,a->id);
	sqlite3_step(stmt);
	rc=sqlite3_column_int(stmt,0);
	if(a->date!=25 || rc!=3)
	{
		++ret;
		g_message("ERROR: expected 25,3 got %d,%d",a->date,rc);
	}
	g_object_unref(a);
	g_ptr_array_free(tarts,TRUE);;
	sqlite3_finalize(stmt);
	if(ret!=0)
	{
		g_message("FAILED: wrong article data");
		return 1;
	}
	g_message("PASSED: article add");
	
	g_print("11 sec. update delay\n");
	g_main_context_iteration(NULL,FALSE);
	g_usleep( G_USEC_PER_SEC*11);
	g_main_context_iteration(NULL,FALSE);
	
	tarts=group_get_all_articles(group,NULL);
	if(tarts->len!=3)
		g_message("wrong list length");
	a=ARTICLE(g_ptr_array_index(tarts,0));
	if(article_is_new(a) || article_is_read(a) || article_is_flagged(a))
		++ret;
	g_object_unref(a);
	a=ARTICLE(g_ptr_array_index(tarts,1));
	if(!article_is_new(a) || !article_is_read(a) || article_is_flagged(a))
		++ret;
	g_object_unref(a);
	a=ARTICLE(g_ptr_array_index(tarts,2));
	if(!article_is_new(a) || article_is_read(a) || !article_is_flagged(a))
		++ret;
	g_object_unref(a);
	g_ptr_array_free(tarts,TRUE);
	if(ret!=0)
	{
		g_message("FAILED: article update failed");
		return 1;
	}
	g_message("PASSED: article update");
	
	serverlist_remove_server(server);
	grps=vserver_get_groups(0);
	if(grps->len!=0)
	{
		ret++;
		g_message("Error: expected 0 groups got %d",grps->len);
	}
	sq_res=sqlite3_prepare(db,"SELECT count(*) FROM article WHERE refcnt!=0;",-1,&stmt,NULL);
	SQL_PREP_VALID(sq_res,db);
	sqlite3_step(stmt);
	rc=sqlite3_column_int(stmt,0);
	sqlite3_finalize(stmt);
	if(rc!=0)
	{
		++ret;
		g_message("Error: article ref counts not correct");
	}
	if(ret!=0)
	{
		g_message("FAILED: server delete");
		return ret;
	}
	g_message("PASSED: server delete (groups deleted, ref counts lowered).");
	g_ptr_array_free(grps,TRUE);
	
	vserver_shutdown();
	serverlist_shutdown();
	if(ret)
		return ret;
	
	return ret;
}


int main(void)
{
	int ret=0;
	
	init_pan();
	ret=test_article();
	
	return ret;
}

