#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>


#include <pan/base/base-prefs.h>
#include <pan/base/util-file.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/vserver.h>
#include <pan/base/group.h>
#include <pan/base/group-private.h>
#include <pan/base/db.h>
#include <pan/base/db-add.h>
#include <gmime/gmime.h>
#include <pan/base/debug.h>

static void init_pan(void)
{
 	char * base;
	char * data_dir;
	char *fn;

	g_mime_init(0);
	base = getenv ("srcdir");
	base = g_strdup_printf ("%s", base!=NULL ? base : getenv("PWD"));
	fn=g_strdup_printf("%s/data/pan.sqlite3",base);
	unlink(fn);
	data_dir = g_strdup_printf ("PAN_HOME=%s/data/", base);
	//g_print("data_dir= %s\n",data_dir);
	putenv (data_dir);
	base_prefs_init (g_get_tmp_dir(), 100, FALSE, FALSE);

	g_free(fn);
	g_free (data_dir);
	g_free (base);
}


static Server* get_server(void)
{
	const char *name="test-group-serveer";
	PString pname=PSTRING_INIT;
	pname=pstring_shallow(name,strlen(name));
	Server *s;
	s=serverlist_get_named_server(&pname);
	if(!s)
	{
		GError *err=NULL;
		s=server_new();
		pstring_copy(&s->name,&pname);
		pstring_set(&s->address,"foobar",6);
		serverlist_add_server(s,&err);
		if(err)
		{
			g_message("Error: %s",err->message);
			g_error_free(err);
		}
	}
	return s;
}

static int test_group(void)
{
	int ret=0,i;
	const char *names[]={"group 1","group 2","group 3"};
	const char *descs[]={"d1","no bodys home",""};
	const int len=3;
	group_add_s *gas;
	GPtrArray *grps=g_ptr_array_sized_new(len);
	Group *group;
	Server *server;
	//set_debug_level(DEBUG_TRACE);
	serverlist_init();
	vserver_init();
	
	server=get_server();
	
	for(i=0;i<len;i++)
	{
		gas=g_new0(group_add_s,1);
		gas->name=pstring_shallow(names[i],strlen(names[i]));
		gas->description=pstring_shallow(descs[i],strlen(descs[i]));
		gas->permision='n';
		g_ptr_array_add(grps,gas);
	}
	g_message("Adding groups");
	add_groups((const group_add_s**)grps->pdata,len,server->id);
	g_message("Groups added");
	for(i=0;i<len;i++)
	{
		gas=g_ptr_array_index(grps,i);
		g_free(gas);
	}
	g_ptr_array_free(grps,TRUE);
	g_message("Getting NEW groups");
	grps=vserver_get_groups(2); //get NEW groups
	if(grps->len!=3)
	{
		ret++;
		g_message("Error: expected 3 groups got %d",grps->len);
	}
	for(i=0;i<grps->len;i++)
	{
		group=GROUP(g_ptr_array_index(grps,i));
		if(g_ascii_strcasecmp(names[i],group_get_name(group)->str)!=0 ||
			g_ascii_strcasecmp(descs[i],group_get_description(group))!=0)
		{
			ret++;
			g_message("Error: wrong data found: $s %s",group_get_name(group),group_get_description(group));
		}
	}
	if(ret!=0)
	{
		for(i=0;i<grps->len;i++)
		{
			group=GROUP(g_ptr_array_index(grps,i));
			g_object_unref(group);
		}
		g_ptr_array_free(grps,TRUE);
	}
	
	g_message("PASSED: group add");
	
	group=GROUP(g_ptr_array_index(grps,0));
	group->article_read_qty=100;
	group->_priv->new=0;
	group_set_dirty(group);
	g_object_unref(group);
	group=GROUP(g_ptr_array_index(grps,1));
	group_inc_read(group,20);
	g_object_unref(group);
	group=GROUP(g_ptr_array_index(grps,2));
	group->article_read_qty=300;
	group_set_dirty(group);
	g_object_unref(group);
	g_ptr_array_free(grps,TRUE);
	
	g_print("11 sec. update delay\n");
	g_main_context_iteration(NULL,FALSE);
	g_usleep( G_USEC_PER_SEC*11);
	g_main_context_iteration(NULL,FALSE);
	
	grps=vserver_get_groups(0); //get ALL groups
	if(grps->len!=3)
	{
		ret++;
		g_message("Error: expected 3 groups got %d",grps->len);
		return ret;
	}
	group=GROUP(g_ptr_array_index(grps,0));
	if(group->article_read_qty!=100 || group_is_new(group))
	{
		ret++;
		g_message("Error %s %s %d",group_get_name(group),group_is_new(group)?"new":"old",group->article_read_qty);
	}
	g_object_unref(group);
	group=GROUP(g_ptr_array_index(grps,1));
	if(!group_is_new(group) || group->article_read_qty!=20)
	{
		ret++;
		g_message("Error %s %s %d",group_get_name(group),group_is_new(group)?"new":"old",group->article_read_qty);
	}
	remove_group(group->id,server->id);
	g_object_unref(group);
	group=GROUP(g_ptr_array_index(grps,2));
	if(group->article_read_qty!=300 || !group_is_new(group))
	{
		ret++;
		g_message("Error %s %s %d",group_get_name(group),group_is_new(group)?"new":"old",group->article_read_qty);
	}
	g_object_unref(group);
	g_ptr_array_free(grps,TRUE);
	g_message("PASSED: group update");
	
	grps=vserver_get_groups(0);
	if(grps->len!=2)
	{
		ret++;
		g_message("Error: expected 2 groups got %d",grps->len);
		return ret;
	}
	g_message("PASSED: group delete");
	g_ptr_array_free(grps,TRUE);
	
	serverlist_remove_server(server);
	grps=vserver_get_groups(0);
	if(grps->len!=0)
	{
		ret++;
		g_message("Error: expected 0 groups got %d",grps->len);
		return ret;
	}
	g_message("PASSED: server delete (groups deleted).");
	g_ptr_array_free(grps,TRUE);
	
	vserver_shutdown();
	serverlist_shutdown();
	if(ret)
		return ret;
	
	return ret;
}


int main(void)
{
	int ret=0;
	
	init_pan();

	ret=test_group();
	
	return ret;
}

