#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>


#include <pan/base/base-prefs.h>
#include <pan/base/util-file.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/db.h>
#include <gmime/gmime.h>
#include <pan/base/debug.h>

static void init_pan(void)
{
 	char * base;
	char * data_dir;
	char *fn;

	g_mime_init(0);
	base = getenv ("srcdir");
	base = g_strdup_printf ("%s", base!=NULL ? base : getenv("PWD"));
	fn=g_strdup_printf("%s/data/pan.sqlite3",base);
	unlink(fn);
	data_dir = g_strdup_printf ("PAN_HOME=%s/data/", base);
	//g_print("data_dir= %s\n",data_dir);
	putenv (data_dir);
	base_prefs_init (g_get_tmp_dir(), 100, FALSE, FALSE);

	g_free(fn);
	g_free (data_dir);
	g_free (base);
}


int
main (void)
{
	Server *s=NULL;
	const char *name="test-server";
	const char *uname="no user name";
	const char *paswd="drowssap";
	const char *sname="news.foobar";
	PString pname,puname,ppaswd,psname;;
	int id=0,ret=0;
	GError *error=NULL;
	GPtrArray *servers=g_ptr_array_new();
	
	pname=puname=ppaswd=psname=PSTRING_INIT;
	pname=pstring_shallow(name,strlen(name));
	puname=pstring_shallow(uname,strlen(uname));
	psname=pstring_shallow(sname,strlen(sname));
	ppaswd=pstring_shallow(paswd,strlen(paswd));
	
	//set_debug_level(DEBUG_TRACE|DEBUG_PAN_OBJECT);
	init_pan();
	serverlist_init();
	g_message("serverlist initialized");
	
	serverlist_get_servers(servers);
	g_message("%d existing servers.",servers->len);
	int i;
	for(i=0;i<servers->len;i++)
	{
		s=SERVER(g_ptr_array_index(servers,i));
		g_message("Existing server %d %s",s->id,s->name.str);
	}
	s=NULL;
	g_ptr_array_free(servers,TRUE);
	
	g_message("checking for old copy of server %s",name);
	s=serverlist_get_named_server(&pname);
	if(s)
	{
		g_message("removing old copy of server %d %s",s->id,s->name.str);
		id=s->id;
		serverlist_remove_server(s);
	}
	g_message("creating new server");
	s=server_new();
	s->rank=2;
	s->port=321;
	s->need_auth=TRUE;
	pstring_copy(&s->username,&puname);
	pstring_copy(&s->password,&ppaswd);
	pstring_copy(&s->address,&psname);
	pstring_copy(&s->name,&pname);
	g_message("adding new server");
	serverlist_add_server(s,&error);
	if(error)
	{
		ret=1;
		g_message("Error adding server: %s\n",error->message);
	}
	g_message("new server added");
	serverlist_shutdown();
	
	serverlist_init();
	g_message("serverlist re-initialized");
	
	s=serverlist_get_named_server(&pname);
	if(!s)
	{
		ret=1;
		g_message("Error server %s not found\n",pname.str);
		goto END;
	}
	if(s->need_auth!=TRUE || !pstring_equal(&puname,&s->username) ||
		!pstring_equal(&psname,&s->address) || !pstring_equal(&ppaswd,&s->password) ||
		s->rank!=2 || s->port!=321)
	{
		ret=1;
		g_message("Error server not saved correctly\n");
		g_message("id: %d name: %s need_auth: %d rank: %d\n",s->id,s->name.str,s->need_auth,s->rank);
		g_message("username: %s password: %s port: %d\n",s->username.str,s->password.str,s->port);
	}
	
	g_message("Altering server.");
	s->port=5000;
	const char *nname="new test server";
	PString pnname=pstring_shallow(nname,strlen(nname));
	server_set_name(s,&pnname,NULL);
	serverlist_shutdown();
	serverlist_init();
	s=serverlist_get_named_server(&pnname);
	if(!s)
	{
		ret=1;
		g_message("Error server %s not found\n",pnname.str);
		goto END;
	}
	if(s->port!=5000)
	{
		ret=1;
		g_message("Error server not updated correctly\n");
		g_message("id: %d name: %s need_auth: %d rank: %d\n",s->id,s->name.str,s->need_auth,s->rank);
		g_message("username: %s password: %s port: %d\n",s->username.str,s->password.str,s->port);
	}
	
	g_message("Deleteing server.");
	serverlist_remove_server(s);
	serverlist_shutdown();
	serverlist_init();
	s=serverlist_get_named_server(&pnname);
	if(s)
	{
		ret=1;
		g_message("Error server not deleted.");
		g_message("id: %d name: %s need_auth: %d rank: %d\n",s->id,s->name.str,s->need_auth,s->rank);
		g_message("username: %s password: %s port: %d\n",s->username.str,s->password.str,s->port);
	}
	
	END:
	serverlist_shutdown();
	close_db();
	return ret;
}
