#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>


#include <pan/base/base-prefs.h>
#include <pan/base/util-file.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/vserver.h>
#include <pan/base/group.h>
#include <pan/base/group-private.h>
#include <pan/base/article.h>
#include <pan/base/db.h>
#include <pan/base/db-add.h>
#include <gmime/gmime.h>
#include <pan/base/debug.h>
#include <pan/base/article-thread.h>
#include <pan/base/pan-base-signals.h>
#include <sqlite3.h>

static void init_pan(void)
{
 	char * base;
	char * data_dir;
	char *fn;

	g_mime_init(0);
	base = getenv ("srcdir");
	base = g_strdup_printf ("%s", base!=NULL ? base : getenv("PWD"));
	fn=g_strdup_printf("%s/data/pan.sqlite3",base);
	unlink(fn);
	data_dir = g_strdup_printf ("PAN_HOME=%s/data/", base);
	//g_print("data_dir= %s\n",data_dir);
	putenv (data_dir);
	base_prefs_init (g_get_tmp_dir(), 100, FALSE, FALSE);

	g_free(fn);
	g_free (data_dir);
	g_free (base);
}


static Server* get_server(void)
{
	const char *name="test-thread-serveer";
	PString pname=PSTRING_INIT;
	pname=pstring_shallow(name,strlen(name));
	Server *s;
	s=serverlist_get_named_server(&pname);
	if(!s)
	{
		GError *err=NULL;
		s=server_new();
		pstring_copy(&s->name,&pname);
		pstring_set(&s->address,"foobar",6);
		serverlist_add_server(s,&err);
		if(err)
		{
			g_message("Error: %s",err->message);
			g_error_free(err);
		}
	}
	return s;
}

static GPtrArray* do_add_groups(void)
{
	int i;
	const char *names[]={"group 1"};
	const char *descs[]={"d1"};
	const int len=1;
	group_add_s *gas;
	GPtrArray *grps=g_ptr_array_sized_new(len);
	Group *group;
	Server *server;
	
	server=get_server();
	
	for(i=0;i<len;i++)
	{
		gas=g_new0(group_add_s,1);
		gas->name=pstring_shallow(names[i],strlen(names[i]));
		gas->description=pstring_shallow(descs[i],strlen(descs[i]));
		gas->permision='n';
		g_ptr_array_add(grps,gas);
	}
	add_groups((const group_add_s**)grps->pdata,len,server->id);
	for(i=0;i<len;i++)
	{
		gas=g_ptr_array_index(grps,i);
		g_free(gas);
	}
	g_ptr_array_free(grps,TRUE);
	grps=vserver_get_groups(2); //get NEW groups
	if(grps->len!=len)
	{
		g_message("Error: expected %d groups got %d",len,grps->len);
	}

	return grps;
}

static GPtrArray* make_articles(void)
{
	GPtrArray *ret=g_ptr_array_sized_new(15);
	article_add_s *a=NULL;
	
	// simple multipart
	
	a=g_new0(article_add_s,1);
	a->uid=1;
	a->date=25;
	a->byte_qty=10;
	a->lines=1;
	a->subject=pstring_shallow("Some multipart part 0123.par (01/04)",-1);
	a->author=pstring_shallow("hello kitty <hkitty@no.server>",-1);
	a->msg_id=pstring_shallow("<12345@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=13;
	a->date=25;
	a->byte_qty=10;
	a->lines=1;
	a->subject=pstring_shallow("Some multipart part 0123.par (02/04)",-1);
	a->author=pstring_shallow("hello kitty <hkitty@no.server>",-1);
	a->msg_id=pstring_shallow("<12346@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=13;
	a->date=25;
	a->byte_qty=10;
	a->lines=1;
	a->subject=pstring_shallow("Some multipart part 0123.par (03/04)",-1);
	a->author=pstring_shallow("hello kitty <hkitty@no.server>",-1);
	a->msg_id=pstring_shallow("<12347@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=14;
	a->date=25;
	a->byte_qty=10;
	a->lines=1;
	a->subject=pstring_shallow("Some multipart part 0123.par (04/04)",-1);
	a->author=pstring_shallow("hello kitty <hkitty@no.server>",-1);
	a->msg_id=pstring_shallow("<12348@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	// test simalar name with different # parts
	a=g_new0(article_add_s,1);
	a->uid=18;
	a->date=26;
	a->byte_qty=10;
	a->lines=1;
	a->subject=pstring_shallow("Some multipart part 0123.par (02/512)",-1);
	a->author=pstring_shallow("hello kitty <hkitty@no.server>",-1);
	a->msg_id=pstring_shallow("<12355@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);

	// thread by ref
	a=g_new0(article_add_s,1);
	a->uid=2;
	a->date=27;
	a->byte_qty=100;
	a->lines=10;
	a->subject=pstring_shallow("Hello",-1);
	a->author=pstring_shallow("\"hello kitty\" <h.kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<pan.2001.01.01.12.59.59@sea.org>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=21;
	a->date=37;
	a->byte_qty=100;
	a->lines=10;
	a->subject=pstring_shallow("RE: Hello",-1);
	a->author=pstring_shallow("\"kitty\" <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<pan.2001.01.01.13.00.01@sea.org>",-1);
	a->refs=pstring_shallow("<pan.2001.01.01.12.59.59@sea.org>",-1);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=22;
	a->date=47;
	a->byte_qty=100;
	a->lines=10;
	a->subject=pstring_shallow("RE: Hello",-1);
	a->author=pstring_shallow("\"hello kitty\" <h.kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<pan.2001.01.01.13.00.02@sea.org>",-1);
	a->refs=pstring_shallow("<pan.2001.01.01.12.59.59@sea.org> <pan.2001.01.01.13.00.01@sea.org>",-1);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=23;
	a->date=57;
	a->byte_qty=100;
	a->lines=10;
	a->subject=pstring_shallow("RE: Hello",-1);
	a->author=pstring_shallow("\"goodbye kitty\" <g.kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<pan.2001.01.01.13.00.03@sea.org>",-1);
	a->refs=pstring_shallow("<pan.2001.01.01.12.59.59@sea.org> <pan.2001.01.01.13.00.01@sea.org>",-1);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	// thread by subject
	a=g_new0(article_add_s,1);
	a->uid=24;
	a->date=57;
	a->byte_qty=100;
	a->lines=10;
	a->subject=pstring_shallow("RE: Hello",-1);
	a->author=pstring_shallow("\"bye kitty\" <b.kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<pan.2001.01.01.13.00.13@sea.org>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);

	// Regression test for bugzilla #8275
	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("some text (1of2).part01.P01 (1/3)",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31425@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("some text (1of2).part01.P01 (2/3)",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31426@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("some text (1of2).part01.P01 (3/3)",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31427@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("some text (2of2).part01.P01 (1/3)",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31435@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("some text (2of2).part01.P01 (2/3)",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31436@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	a=g_new0(article_add_s,1);
	a->uid=3;
	a->date=35;
	a->byte_qty=102;
	a->lines=20;
	a->subject=pstring_shallow("some text (2of2).part01.P01 (3/3)",-1);
	a->author=pstring_shallow("kitty <kitty@no.server>",-1);
	a->msg_id=pstring_shallow("<31437@no>",-1);
	a->refs=pstring_shallow("",0);
	a->xref=pstring_shallow("",0);
	g_ptr_array_add(ret,a);
	
	return ret;
}

static void do_unref(gpointer a, gpointer ud)
{
	g_object_unref(ARTICLE(a));
}

static int test_simple_multi(Article *a)
{
	int ret=0;
	
	if(a->parts!=4)
	{
		++ret;
		g_message("Wrong total parts count. expected 4 got %d",a->parts);
	}
	if(a->multipart_state!=MULTIPART_STATE_ALL)
	{
		++ret;
		g_message("Wrong multipart state. expected %d got %d",MULTIPART_STATE_ALL,a->multipart_state);
	}
	if(article_get_num_parts(a)!=4)
	{
		++ret;
		g_message("Wrong number of parts. expected 4 got %d",article_get_num_parts(a));
	}
	
	if(ret!=0)
		g_message("FAILED: test_simple_multipart");
	else
		g_message("PASS: test_simple_multipart");
	
	return ret;
}

static int test_multi2(Article *a)
{
	int ret=0;
	GArray *array;
	
	if(a->parts!=512)
	{
		++ret;
		g_message("Wrong total parts count. expected 512 got %d",a->parts);
	}
	if(a->multipart_state!=MULTIPART_STATE_SOME)
	{
		++ret;
		g_message("Wrong multipart state. expected %d got %d",MULTIPART_STATE_SOME,a->multipart_state);
	}
	if(article_get_num_parts(a)!=1)
	{
		++ret;
		g_message("Wrong number of parts. expected 1 got %d",article_get_num_parts(a));
	}
	array=article_get_part_nums(a);
	if(array->len!=1)
	{
		++ret;
		g_message("Error: wrong length of parts listing expected 1 got %d",array->len);
	}
	if(g_array_index(array,gint16,0)!=2)
	{
		++ret;
		g_message("Error: wrong part number expected 2 got %d",g_array_index(array,gint16,0));
	}
	g_array_free(array,TRUE);
	array=article_get_missing_part_nums(a);
	if(array->len!=511)
	{
		++ret;
		g_message("Error: wrong number of missing parts expected 511 got %d",array->len);
	}
	if(ret!=0)
		g_message("FAILED: test_multi2");
	else
		g_message("PASS: test_multi2");
	
	return ret;
}

static int test_ref_thread(Article *a)
{
	int ret=0;
	GSList *arts=article_get_children(a);
	
	if(a->parts!=0)
	{
		++ret;
		g_message("Wrong parts count. expected 0 got %d",a->parts);
	}
	if(a->multipart_state!=MULTIPART_STATE_NONE)
	{
		++ret;
		g_message("Wrong multipart state. expected %d got %d",MULTIPART_STATE_NONE,a->multipart_state);
	}
	if(article_get_num_children(a)!=2)
	{
		++ret;
		g_message("Wrong number of children. expected 2 got %d",article_get_num_children(a));
	}else
	{
		Article *a2=ARTICLE(arts->data);
		if(a2->date!=37)
			a2=ARTICLE(arts->next->data);
		if(article_get_num_children(a2)!=2)
		{
			++ret;
			g_message("Wrong number of L2 children. expected 2 got %d",article_get_num_children(a2));
		}	
		// for missing article test
		article_delete(a2);
	}
	
	if(ret!=0)
		g_message("FAILED: test_ref_thread");
	else
		g_message("PASS: test_ref_thread");
	
	g_slist_free(arts);
	return ret;
}

static int test_bug_8275(Article *a)
{
	int ret=0;
	

	if(a->parts!=3)
	{
		++ret;
		g_message("Wrong total parts count. expected 3 got %d",a->parts);
	}
	if(a->multipart_state!=MULTIPART_STATE_ALL)
	{
		++ret;
		g_message("Wrong multipart state. expected %d got %d",MULTIPART_STATE_ALL,a->multipart_state);
	}
	if(article_get_num_parts(a)!=3)
	{
		++ret;
		g_message("Wrong number of parts. expected 3 got %d",article_get_num_parts(a));
	}
	
	if(ret!=0)
		g_message("FAILED: test_bug_8275");
	else
		g_message("PASS: test_bug_8275");
	return ret;
}

static int test_missing(Article *a)
{
	int ret=0;
	
	if(a->parts!=0)
	{
		++ret;
		g_message("Wrong parts count. expected 0 got %d",a->parts);
	}
	if(a->multipart_state!=MULTIPART_STATE_NONE)
	{
		++ret;
		g_message("Wrong multipart state. expected %d got %d",MULTIPART_STATE_NONE,a->multipart_state);
	}
	if(article_get_num_children(a)!=3)
	{
		++ret;
		g_message("Wrong number of children. expected 3 got %d",article_get_num_children(a));
	}
	
	if(ret!=0)
		g_message("FAILED: test_missing");
	else
		g_message("PASS: test_missing");
	
	return ret;
}

int main(void)
{
	init_pan();
//	set_debug_level(DEBUG_TRACE);
	int ret=0,i,rc;
	GPtrArray *grps,*arts,*tarts;
	Server *server;
	Group *group;
	Article *a;
	sqlite3 *db=get_db();
	sqlite3_stmt *stmt;
	int sq_res;
	
	serverlist_init();
	vserver_init();
	server=get_server();
	grps=do_add_groups();
	arts=make_articles();
	
	group=g_ptr_array_index(grps,0);
	add_articles(group->id,server->id,(const article_add_s**)arts->pdata,arts->len);
	for(i=0;i<arts->len;i++)
	{
		g_free(g_ptr_array_index(arts,i));
	}
	g_ptr_array_free(arts,TRUE);
	arts=NULL;
	thread_group(group->id);
	
	// check number of top articles
	tarts=group_get_all_articles(group,NULL);
	if(tarts->len!=9)
	{
		g_message("FAILED: adding articles expected 9 got %d",tarts->len);
		++ret;
	}
	char buf[1024];
	// other checks
	for(i=0;i<tarts->len;++i)
	{
		a=ARTICLE(g_ptr_array_index(tarts,i));
		
		if(article_get_parent(a)==NULL && article_get_group(a)!=NULL)
		{
			if(a->date==25)
				ret+=test_simple_multi(a);
			else if(a->date==26)
				ret+=test_multi2(a);
			else if(a->date==27)
				ret+=test_ref_thread(a);
			else
				ret+=test_bug_8275(a);
		}
	}
	g_ptr_array_foreach(tarts,do_unref,NULL);
	g_ptr_array_free(tarts,TRUE);
	
	thread_group(group->id);
	
	// check number of top articles
	tarts=group_get_all_articles(group,NULL);
	if(tarts->len!=8)
	{
		g_message("FAILED: expected 8 articles got %d",tarts->len);
		++ret;
	}
	char subj[100];
	for(i=0;i<tarts->len;++i)
	{
		a=ARTICLE(g_ptr_array_index(tarts,i));
		article_get_subject(a,subj,100);
		if(g_ascii_strcasecmp(subj,"Hello")==0)
			ret+=test_missing(a);
		g_object_unref(a);
	}
	g_ptr_array_free(tarts,TRUE);
	
	if(ret==0)
		g_message("PASSED: test-thread");
	else
		g_message("FAILED: test-thread");
	
	g_object_unref(group);
	g_ptr_array_free(grps,TRUE);
	
	vserver_shutdown();
	serverlist_shutdown();
	pan_base_signals_shutdown(pan_base_signals_get());
	return ret;
}


