#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>


#include <pan/base/base-prefs.h>
#include <pan/base/util-file.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/vserver.h>
#include <pan/base/group.h>
#include <pan/base/group-private.h>
#include <pan/base/db.h>
#include <pan/base/db-add.h>
#include <gmime/gmime.h>
#include <pan/base/debug.h>

static void init_pan(void)
{
 	char * base;
	char * data_dir;
	char *fn;

	g_mime_init(0);
	base = getenv ("srcdir");
	base = g_strdup_printf ("%s", base!=NULL ? base : getenv("PWD"));
	fn=g_strdup_printf("%s/data/pan.sqlite3",base);
	unlink(fn);
	data_dir = g_strdup_printf ("PAN_HOME=%s/data/", base);
	//g_print("data_dir= %s\n",data_dir);
	putenv (data_dir);
	base_prefs_init (g_get_tmp_dir(), 100, FALSE, FALSE);

	g_free(fn);
	g_free (data_dir);
	g_free (base);
}

static int test_vserver(void)
{
	int ret=0,i;
	const char *names[]={"vgroup 1","vgroup 2","vgroup 3"};
	const char *descs[]={"","no home","boo"};
	const int len=3;
	const char *vname="test vserver";
	int vid=-1;
	group_add_s *gas;
	GPtrArray *grps=g_ptr_array_sized_new(len);
	const GPtrArray *vs;
	Group *group;
	
	vserver_init();

	if(!vserver_add(vname))
		g_message("somethings wrong");
	vs=vserver_get_list();
	i=0;
	while(i<vs->len && vid==-1)
	{
		vserver *v=g_ptr_array_index(vs,i);
		++i;
		if(strcmp(vname,v->name)==0)
			vid=v->id;
	}
	if(vid==-1)
	{
		g_message("FAILED: adding vserver.");
		return 1;
	}
	
	for(i=0;i<len;i++)
	{
		gas=g_new0(group_add_s,1);
		gas->name=pstring_shallow(names[i],strlen(names[i]));
		gas->description=pstring_shallow(descs[i],strlen(descs[i]));
		gas->permision='n';
		grps->pdata[i]=gas;
	}
	add_groups((const group_add_s**)grps->pdata,len,20);
	for(i=0;i<len;i++)
	{
		gas=g_ptr_array_index(grps,i);
		g_free(gas);
	}
	g_ptr_array_free(grps,TRUE);
	
	grps=vserver_get_groups(2);
	if(grps->len!=3)
	{
		g_message("FAILED: adding groups.");
		return 1;
	}
	vserver_add_groups(vid,grps);
	for(i=0;i<len;i++)
		g_object_unref(GROUP(g_ptr_array_index(grps,i)));
	g_ptr_array_free(grps,TRUE);
	grps=vserver_get_groups(vid);
	if(grps->len!=3)
	{
		g_message("FAILED: adding groups to new vserver.");
		return 1;
	}
	for(i=0;i<len;i++)
		g_object_unref(GROUP(g_ptr_array_index(grps,i)));
	g_ptr_array_free(grps,TRUE);
	g_message("PASSED: vserver.");
	vserver_shutdown();
	return 0;
}

int main(void)
{
	int ret=0;
	
	init_pan();

	ret=test_vserver();
	
	return ret;
}

